/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CThemeImageCollection.h  -  description
                             -------------------
    begin                : Sat Jun 28 2008
    copyright            : (C) 2003-2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, 
   this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, 
   this list of conditions and the following disclaimer in the documentation 
   and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
   may be used to endorse or promote products derived from this software 
   without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CTHEMEIMAGECOLLECTION_H__
#define	__CTHEMEIMAGECOLLECTION_H__

#include <sptk3/cxml>
#include <sptk3/gui/CPngImage.h>
#include <sptk3/gui/CThemeImageState.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief A collection of theme images
///
/// The images are stored for the different states of the object
class CThemeImageCollection {
    CPngImage*  m_images[MAX_IMAGE_STATES];         ///< List of images for different image states
    CPngImage*  m_overlayImages[MAX_IMAGE_STATES];  ///< List of overlay images for different image states
    int         m_border[4];                        ///< Border, for the first image
    bool        m_stretch;                          ///< Stretch flag, for the first image
public:
    /// @brief Default constructor
    CThemeImageCollection();

    /// @brief Destructor
    ~CThemeImageCollection() { clear(); }

    /// @brief Clears the collection
    void clear();

    /// @brief Loads the collection from SPTK theme
    void loadFromSptkTheme(const CStrings& objectNames);

    /// @brief Loads the collection from GTK theme
    void loadFromGtkTheme(CXmlDoc& gtkTheme,std::string imagesXPath,std::string attribute="",std::string attributeValue="");

    /// @brief Returns border for a paticular index (0..3)
    int border(int ndx) const { return m_border[ndx]; }

    /// @brief Returns borders array
    int* border() { return m_border; }

    /// @brief Returns draw stretch flag
    bool stretch() const { return m_stretch; }

    /// @brief Returns an image for a particular state
    CPngImage* image(CThemeImageState state) const; 

    /// @brief Returns an overlay image for a particular state
    CPngImage* overlayImage(CThemeImageState state) const;

    /// @brief Returns full file name from theme file name
    static std::string gtkFullFileName(std::string fileName);
    
    /// @brief Returns array of border values from an XML border attribute
    ///
    /// Fills in the border values, or fills in border with zeros.
    /// @param node const CXmlNode*, a node with border attribute
    /// @param border int[4], output border values
    /// @returns true if success
    static bool getBorder(const CXmlNode* node,int border[4]);
};

/// @}
}

#endif

