/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CTabImage.cpp  -  description
                             -------------------
    begin                : Thu Nov 09 2006
    copyright            : (C) 2003-2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/fl_draw.h>

#include <sptk3/gui/CTabImage.h>
#include <sptk3/gui/CThemeImageCollection.h>

using namespace std;
using namespace sptk;

CTabImage::CTabImage(const CTar& tar,const CXmlNode* tabImageNode) {
    m_name = tabImageNode->getAttribute("name");
    string fileName = tabImageNode->getAttribute("image");
    m_image = new CPngImage(tar.file(fileName));
    m_leftFrameWidth = tabImageNode->getAttribute("left_frame","0");
    m_rightFrameWidth = tabImageNode->getAttribute("right_frame","0");
    m_topFrameHeight = tabImageNode->getAttribute("top_frame","0");
    m_bottomFrameHeight = tabImageNode->getAttribute("bottom_frame","0");
    if (tabImageNode->getAttribute("fill") == "stretch")
        m_backgroundDrawMode = CPngImage::PDM_STRETCH;
    else
        m_backgroundDrawMode = CPngImage::PDM_TILE;
}

CTabImage::CTabImage(const CXmlNode* tabImageNode) {
    string state = tabImageNode->getAttribute("state");
    if (state == "NORMAL")
        m_name = "active";
    else
        m_name = "inactive";
    string fileName = CThemeImageCollection::gtkFullFileName(tabImageNode->getAttribute("file"));
    m_image = new CPngImage(fileName);
    int border[4];
    CThemeImageCollection::getBorder(tabImageNode,border);
    m_leftFrameWidth = border[0];
    m_rightFrameWidth = border[1];
    m_topFrameHeight = border[2];
    m_bottomFrameHeight = border[3];
    m_backgroundDrawMode = CPngImage::PDM_STRETCH;
}

void CTabImage::draw(int x,int y,int w,int h) {
    /// Top left corner
    m_image->draw(x,y,m_leftFrameWidth,m_topFrameHeight,0,0);

    /// Bottom left corner
    m_image->draw(x,y+h-m_bottomFrameHeight,m_leftFrameWidth,m_bottomFrameHeight,0,m_image->h()-m_bottomFrameHeight);

    /// Top right corner
    m_image->draw(x+w-m_rightFrameWidth,y,m_rightFrameWidth,m_topFrameHeight,m_image->w()-m_rightFrameWidth,0);

    /// Bottom right corner
    m_image->draw(x+w-m_rightFrameWidth,y+h-m_bottomFrameHeight,m_rightFrameWidth,m_bottomFrameHeight,m_image->w()-m_rightFrameWidth,m_image->h()-m_bottomFrameHeight);

    /// Left side
    m_image->cutStretchDraw(0,m_topFrameHeight,m_leftFrameWidth,m_image->h()-(m_topFrameHeight+m_bottomFrameHeight),
                            x,y+m_topFrameHeight,m_leftFrameWidth,h-(m_topFrameHeight+m_bottomFrameHeight));

    /// Top side
    m_image->cutStretchDraw(m_leftFrameWidth,0,m_image->w()-(m_leftFrameWidth+m_rightFrameWidth),m_topFrameHeight,
                            x+m_leftFrameWidth,y,w-(m_leftFrameWidth+m_rightFrameWidth),m_topFrameHeight);

    /// Right side
    m_image->cutStretchDraw(m_image->w()-m_leftFrameWidth,m_topFrameHeight,m_leftFrameWidth,m_image->h()-(m_topFrameHeight+m_bottomFrameHeight),
                            x+w-m_leftFrameWidth,y+m_topFrameHeight,m_leftFrameWidth,h-(m_topFrameHeight+m_bottomFrameHeight));

    /// Bottom side
    m_image->cutStretchDraw(m_leftFrameWidth,m_image->h()-m_topFrameHeight,m_image->w()-(m_leftFrameWidth+m_rightFrameWidth),m_topFrameHeight,
                            x+m_leftFrameWidth,y+h-m_topFrameHeight,w-(m_leftFrameWidth+m_rightFrameWidth),m_topFrameHeight);

    /// Background
    m_image->cutStretchDraw(m_leftFrameWidth,m_topFrameHeight,m_image->w()-(m_leftFrameWidth+m_rightFrameWidth),m_image->h()-(m_topFrameHeight+m_bottomFrameHeight),
                            x+m_leftFrameWidth,y+m_topFrameHeight,w-(m_leftFrameWidth+m_rightFrameWidth),h-(m_topFrameHeight+m_bottomFrameHeight));
}

void CTabImages::loadSptkTabs(const CTar& tar,const CXmlNode* tabImagesNode) {
    clear();
    CXmlNode::const_iterator itor = tabImagesNode->begin();
    for (; itor != tabImagesNode->end(); itor++) {
        CXmlNode*   tabNode = *itor;
        CTabImage*  tabImage = new CTabImage(tar,tabNode);
        (*this)[tabImage->name()] = tabImage;
    }
}

void CTabImages::loadGtkTabs(const CXmlNodeVector& tabImageNodes) {
    clear();
    CXmlNode::const_iterator itor = tabImageNodes.begin();
    for (; itor != tabImageNodes.end(); itor++) {
        CXmlNode*   tabNode = *itor;
        string gapSide = tabNode->getAttribute("gap_side");
        if (gapSide == "BOTTOM") {
            CTabImage*  tabImage = new CTabImage(tabNode);
            (*this)[tabImage->name()] = tabImage;
        }
    }
}

void CTabImages::clear() {
    for (iterator itor = begin(); itor != end(); itor++)
        delete itor->second;
    map<std::string,CTabImage*>::clear();
}

CTabImage* CTabImages::tabImage(const char* imageName) {
    iterator itor = find(imageName);
    if (itor == end())
        return 0;
    return itor->second;
}
