
/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CThemeImageCollection.cpp  -  description
                             -------------------
    begin                : Sat Jun 28 2008
    copyright            : (C) 2003-2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include "ThemeUtils.h"
#include <sptk3/gui/CThemes.h>
#include <sptk3/gui/CThemeImageCollection.h>

#include <iostream>

using namespace std;
using namespace sptk;
   
CThemeImageCollection::CThemeImageCollection() {
    m_stretch = true;
    for (unsigned i = 0; i < MAX_IMAGE_STATES; i++) {
        m_images[i] = 0;
        m_overlayImages[i] = 0;
    }
}

void CThemeImageCollection::clear() {
    for (unsigned i = 0; i < MAX_IMAGE_STATES; i++) {
        if (m_images[i]) {
            delete m_images[i];
            m_images[i] = 0;
        }
        if (m_overlayImages[i]) {
            delete m_overlayImages[i];
            m_overlayImages[i] = 0;
        }
    }
    m_border[0] = 8;
    m_border[1] = 8;
    m_border[2] = 8;
    m_border[3] = 8;
}

CPngImage* CThemeImageCollection::image(CThemeImageState state) const {
    if (m_images[state])
        return m_images[state];
    int bstate = state;
    if (bstate & THMF_HIGHLIGHTED)
        bstate -= THMF_HIGHLIGHTED;
    if (m_images[bstate])
        return m_images[bstate];
    return m_images[0];
}

CPngImage* CThemeImageCollection::overlayImage(CThemeImageState state) const { 
    if (m_overlayImages[state])
        return m_overlayImages[state];
    int bstate = state;
    if (bstate & THMF_HIGHLIGHTED)
        bstate -= THMF_HIGHLIGHTED;
    if (m_overlayImages[bstate])
        return m_overlayImages[bstate];
    return m_overlayImages[0];
}

void CThemeImageCollection::loadFromSptkTheme(const CStrings& objectNames) {
    clear();
    for (unsigned i = 0; i < objectNames.size() && i < MAX_IMAGE_STATES; i++) {
        if (!objectNames[i].empty())
            m_images[i] = loadValidatePNGImage(objectNames[i] + ".png",false);
    }
}

string CThemeImageCollection::gtkFullFileName(string fileName) {
    if (fileName.empty())
        return fileName;
    if (fileName[0] == '/') fileName = fileName.substr(1,255);
    int extPos = lowerCase(fileName).find(".png");
    if (extPos == fileName.length() - 4)
        return CThemes::themeFolder() + fileName;
    return "";
}

void CThemeImageCollection::loadFromGtkTheme(CXmlDoc& gtkTheme,std::string imagesXPath,std::string attribute,std::string attributeValue) {
    static const CStrings buttonStates("NORMAL|ACTIVE|PRELIGHT","|");

    CXmlNodeVector images;

    gtkTheme.select(images,imagesXPath);
    bool borderInitted = false;
    string normalImageFileName;
    string normalOverlayFileName;
    for (CXmlNode::iterator itor = images.begin(); itor != images.end(); itor++) {
        CXmlNode* imageNode = *itor;

        if (!attribute.empty() && imageNode->getAttribute(attribute) != attributeValue)
            continue;

        bool defaultFrame = imageNode->getAttribute("detail","") == "buttondefault";

        string fileName = gtkFullFileName(imageNode->getAttribute("file"));
        string overlayFileName = gtkFullFileName(imageNode->getAttribute("overlay_file"));

        string state = upperCase(imageNode->getAttribute("state","NORMAL"));
        int buttonState = buttonStates.indexOf(state);

        if (normalImageFileName.empty() && (state == "NORMAL" || state == "ACTIVE"))
            normalImageFileName = fileName;

        if (normalOverlayFileName.empty() && (state == "NORMAL" || state == "ACTIVE"))
            normalOverlayFileName = overlayFileName;

        if (!borderInitted) {
            m_stretch = imageNode->getAttribute("stretch") == "TRUE";
            getBorder(imageNode,m_border);
            borderInitted = true;
        }

        string shadow = upperCase(imageNode->getAttribute("shadow","OUT"));
        if (shadow == "ETCHED_IN")
            continue;

        if (defaultFrame)
            buttonState = THM_DEFAULT_FRAME;
        else if (buttonState > -1 && shadow == "IN")
            buttonState |= THMF_ACTIVE;

        if (buttonState > -1) {
            m_images[CThemeImageState(buttonState)] = NULL;
            m_overlayImages[CThemeImageState(buttonState)] = NULL;
            if (!fileName.empty() && fileName.find(".png") != string::npos)
                m_images[CThemeImageState(buttonState)] = loadValidatePNGImage(fileName,true);
            if (!overlayFileName.empty() && overlayFileName.find(".png") != string::npos)
                m_overlayImages[CThemeImageState(buttonState)] = loadValidatePNGImage(overlayFileName,true);
        }
    }

    if (!m_images[THM_IMAGE_NORMAL] && !normalImageFileName.empty())
        m_images[THM_IMAGE_NORMAL] = loadValidatePNGImage(normalImageFileName,true);

    if (!m_overlayImages[THM_IMAGE_NORMAL] && !normalOverlayFileName.empty())
        m_overlayImages[THM_IMAGE_NORMAL] = loadValidatePNGImage(normalOverlayFileName,true);
}

bool CThemeImageCollection::getBorder(const CXmlNode* node,int borders[4]) {
    bool rc = false;
    string border = node->getAttribute("border");
    size_t pos1 = border.find("{");
    size_t pos2 = border.find("}");
    memset(borders,0,sizeof(border));
    if (pos1 != string::npos && pos2 != string::npos ) {
        pos1++;
        border = border.substr(pos1,pos2-pos1);
        CStrings borderStrs(border,",");
        for (unsigned i = 0; i < 4 && i < borderStrs.size(); i++)
            borders[i] = string2int(borderStrs[i]);
        rc = true;
    }
    return rc;
}
