/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBaseMailConnect.cpp  -  description
                             -------------------
    begin                : 25 June 2003
    copyright            : (C) 2000-2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sstream>

#ifdef _MSC_VER
#pragma warning (disable: 4786)
#endif

#include <sptk3/CBase64.h>
#include <sptk3/CBaseMailConnect.h>
#include <sptk3/CStrings.h>
#include <sptk3/CDateTime.h>
#include <sptk3/string_ext.h>
#include <time.h>

#include <map>

#define LINE_CHARS 72

using namespace std;
using namespace sptk;

class CContentTypes
{
        static map<string, string> m_contentTypes;
    public:
        CContentTypes() { if (!m_contentTypes.size()) init(); }
        void init();
        static string type (string fileName);
};

map<string, string> CContentTypes::m_contentTypes;
static CContentTypes contentTypes;

void CContentTypes::init()
{
    m_contentTypes["txt"] = "text/plain";
    m_contentTypes["htm"] = "text/html";
    m_contentTypes["html"] = "text/html";
    m_contentTypes["gif"] = "image/gif";
    m_contentTypes["png"] = "image/png";
    m_contentTypes["bmp"] = "image/bmp";
    m_contentTypes["jpg"] = "image/jpeg";
    m_contentTypes["tif"] = "image/tiff";
    m_contentTypes["pdf"] = "application/pdf";
    m_contentTypes["xls"] = "application/vnd.ms-excel";
    m_contentTypes["csv"] = "text/plain";
    m_contentTypes["doc"] = "application/msword";
    m_contentTypes["wav"] = "application/data";
}

string CContentTypes::type (string fileName)
{
    const char *extension = strrchr (fileName.c_str(), '.');
    if (!extension)
        return "application/octet-stream";
    extension++;
    if (strlen (extension) > 4)
        return "application/octet-stream";
    const map<string, string>::iterator itor = m_contentTypes.find (extension);
    if (itor != m_contentTypes.end()) return itor->second;
    else
        return "application/octet-stream";
}

void CBaseMailConnect::mimeFile (string fileName, string fileAlias, stringstream& message)
{
    CBuffer  bufSource;
    string  strDest;

    bufSource.loadFromFile (fileName.c_str());

    string ctype = contentTypes.type (trim (fileName));

    message << "Content-Type: " << ctype << "; name=\"" << fileAlias << "\"" << endl;
    message << "Content-Transfer-Encoding: base64" << endl;
    message << "Content-Disposition: attachment; filename=\"" << fileAlias << "\"" << endl << endl;

    CBuffer buffer;

    CBase64::encode (strDest, bufSource);
    uint32_t cnt = (uint32_t) strDest.length();
    const char *data = strDest.c_str();
    char line[90];
    for (uint32_t p = 0; p < cnt; p += LINE_CHARS) {
        uint32_t length = cnt - p;
        if (length > LINE_CHARS) length = LINE_CHARS;
        memcpy (line, data + p, 80);
        line[length] = '\n';
        //line[length+1] = '\n';
        length++;
        line[length] = 0;
        buffer.append (line, length);
    }
    message << buffer.data();
}

void CBaseMailConnect::mimeMessage (CBuffer& buffer)
{
    static const char boundary[] = "--MESSAGE-MIME-BOUNDARY--";
    static const char boundary2[] = "--TEXT-MIME-BOUNDARY--";
    stringstream message;

    if (m_from.length())
        message << "From: " << m_from << endl;
    else
        message << "From: postmaster" << endl;

    replaceAll (m_to, ";", ", ");
    message << "To: " << m_to << endl;

    if (m_cc.length()) {
        m_cc = replaceAll(m_cc, ";", ", ");
        message << "CC: " << m_cc << endl;
    }

    message << "Subject: " << m_subject << endl;

    CDateTime   date = CDateTime::Now();
    short dy, dm, dd, th, tm, ts, tms;
    date.decodeDate (&dy, &dm, &dd);
    date.decodeTime (&th, &tm, &ts, &tms);

    char        dateBuffer[128];
    const char* sign = "-";
    int   offset = CDateTime::timeZoneOffset;
    if (offset >= 0)
        sign = "";
    else
        offset = -offset;

    sprintf(dateBuffer,
             "Date: %s, %i %s %04i %02i:%02i:%02i %s%04i (%s)",
             date.dayOfWeekName().substr (0, 3).c_str(),
             dd,
             CDateTime::monthNames[dm-1].substr (0, 3).c_str(),
             dy,
             th, tm, ts,
             sign,
             offset * 100,
             CDateTime::timeZoneName.c_str()
            );

    message << dateBuffer << endl;

    message << "MIME-Version: 1.0" << endl;
    message << "Content-Type: multipart/mixed; boundary=\"" << boundary << "\"" << endl << endl;

    message << endl << "--" << boundary << endl;

    if (m_body.type() == MMT_PLAIN_TEXT_MESSAGE) {
        message << "Content-Type: text/plain; charset=ISO-8859-1" << endl;
        message << "Content-Transfer-Encoding: 7bit" << endl;
        message << "Content-Disposition: inline" << endl << endl;
        message << m_body.text() << endl << endl;
    } else {
        message << "Content-Type: multipart/alternative;  boundary=\"" << boundary2 << "\"" << endl << endl;

        message << endl << "--" << boundary2 << endl;
        message << "Content-Type: text/plain; charset=ISO-8859-1" << endl;
        message << "Content-Disposition: inline" << endl;
        message << "Content-Transfer-Encoding: 8bit" << endl << endl;

        message << m_body.text() << endl << endl;

        message << endl << "--" << boundary2 << endl;
        message << "Content-Type: text/html; charset=ISO-8859-1" << endl;
        message << "Content-Disposition: inline" << endl;
        message << "Content-Transfer-Encoding: 7bit" << endl << endl;

        message << m_body.html() << endl << endl;
        message << endl << "--" << boundary2 << "--" << endl;
    }

    //message << endl << "--" << boundary << "--" << endl;

    CStrings sl (m_attachments, ";");
    for (unsigned i = 0; i < sl.size(); i++) {
        string attachment = sl[i];
        string attachmentAlias = attachment;
        const char *separator = "\\";
        if (attachment.find ("/") != string::npos)
            separator = "/";
        CStrings attachmentParts (attachment, separator);
        uint32_t attachmentPartsCount = (uint32_t) attachmentParts.size();
        if (attachmentPartsCount > 1)
            attachmentAlias = attachmentParts[attachmentPartsCount-1].c_str();
        if (attachment.length()) {
            message << endl << "--" << boundary << endl;
            mimeFile (attachment, attachmentAlias, message);
            //message << "--" << boundary << "--" << endl;
        }
    }

    message << endl << "--" << boundary << "--" << endl;

    buffer.reset(2048);
    buffer.append(message.str().c_str(), (uint32_t) message.str().length());
    buffer.saveToFile("/tmp/mimed.txt");
    //exit(0);
}
